<?php
/*
Plugin Name: Flickr Tag Cloud Widget
Plugin URI: http://wordpress.org/extend/plugins/flickr-tag-cloud-widget/
Donate link: https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=joshgerdes%40gmail%2ecom&item_name=Flickr%20Tag%20Cloud%20Widget%20Wordpress%20Plugin%20Donation&no_shipping=0&no_note=1&tax=0&currency_code=USD&lc=US&bn=PP%2dDonationsBF&charset=UTF%2d8
Description: Display your flickr tags as a tag cloud in your widget sidebar
Version: 1.5
Author: Josh Gerdes
Author URI: http://www.joshgerdes.com
Contributors: joshgerdes
Tags: flickr-tag-cloud-widget, flickr, widget, tag, tags, cloud, tagcloud, sidebar, images, image, photo, photos
Requires at least: 2.0
Tested up to: 2.6.1
Stable tag: 1.5

Copyright (c) 2007-2008
Released under the GPL license
http://www.gnu.org/licenses/gpl.txt
*/
function widget_flickrtagcloud_init()
{	
	// Check for the required API functions
	if ( !function_exists('register_sidebar_widget') || !function_exists('register_widget_control') ) return;

	// Curl replacement for file_get_contents
	function file_get_contents_curl($url) {
        $ch = curl_init();
		curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); //Set curl to return the data instead of printing it to the browser.
		curl_setopt($ch, CURLOPT_URL, $url);
		$data = curl_exec($ch);
		curl_close($ch);
		return $data;
	}

	// Get the basedir url for the plugin
	function get_plugin_basedir_uri() {
		$uri = get_settings('siteurl') . '/wp-content/plugins/'. basename(dirname(__FILE__)) .'/';
		return $uri;
	}
	
	// Global Variables and Defaults
	define('CACHEFILEURL', get_plugin_basedir_uri().'flickrtagcloud.cache');
	define('CACHEFILE', dirname(__FILE__).'/flickrtagcloud.cache');
	define('DEFAULT_CACHETIME', '604800'); //604800 = a week , -1 = disabled
	define('DEFAULT_MINFONTSIZE', '10');
	define('DEFAULT_MAXFONTSIZE', '24');
	define('DEFAULT_MAXCOUNT', '-1');
	define('DEFAULT_TARGET', '_top');

	// data retrieval function
	function get_flickr_data($args) {
	
		extract($args);
			
		$options = get_option('widget_flickrtagcloud');
		$title = $options['title'];
		$flickrid = $options['user'];
		$minfontsize = $options['minfontsize'];
		$maxfontsize = $options['maxfontsize'];
		$api_key= $options['key'];
		$maxcount = $options['maxcount'];
		$cache = $options['cache'];
		$target = $options['target'];
		
		// Defaults	
		if(!$minfontsize or $minfontsize<0) $minfontsize = DEFAULT_MINFONTSIZ;
		if(!$maxfontsize or $maxfontsize<0) $maxfontsize = DEFAULT_MAXFONTSIZ;
		if(!$maxcount or $maxcount<-1) $maxcount = DEFAULT_MAXCOUNT;
		if(!$cache or $cache<-1) $cache = DEFAULT_CACHETIME;
		if(!target) $target = DEFAULT_TARGET;
		if($maxcount==0) $maxcount = DEFAULT_MAXCOUNT;
		
		// Setup properties for API call		
		$params = array(
			'api_key'	=> $api_key,
			'method'	=> 'flickr.people.getInfo',
			'user_id'	=> $flickrid,
			'format'	=> 'php_serial',
		);
		
		// Loop through parameters and encode
		$encoded_params = array();
		foreach ($params as $k => $v){
			// Encode parameters for the url of the API call
			$encoded_params[] = urlencode($k).'='.urlencode($v);
		}
		
		// Call the API and decode the response
		$url = "http://api.flickr.com/services/rest/?".implode('&', $encoded_params);
		
		// Fetch the info
		$rsp = file_get_contents_curl($url);
		$rsp_obj = unserialize($rsp);
		
		// Check if successful and display error if not
		if ($rsp_obj['stat'] == 'ok'){
			// Grab the photo url for the given person
			$photourl = $rsp_obj['person']['photosurl']['_content'];
		}
		
		// Setup properties for API call		
		$params = array(
			'api_key'	=> $api_key,
			'method'	=> 'flickr.tags.getListUserPopular',
			'count'		=> $maxcount,
			'user_id'	=> $flickrid,
			'format'	=> 'php_serial',
		);
		
		$encoded_params = array();
		// Loop through parameters and encode
		foreach ($params as $k => $v){
			// Encode parameters for the url of the API call
			$encoded_params[] = urlencode($k).'='.urlencode($v);
		}

		// Call the API and decode the response
		$url = "http://api.flickr.com/services/rest/?".implode('&', $encoded_params);		

		// Fetch the info
		$rsp = file_get_contents_curl($url);
		$rsp_obj = unserialize($rsp);

		$data .= "\n";
		$data .= $before_widget . $before_title . $title . $after_title;
		$data .= '<div class="flickrtagcloud">';

		// Check if successful and display error if not
		if ($rsp_obj['stat'] == 'ok'){
		
			// Reset the local count
			$count = 0;	
			$mintagcount = 0;
			$maxtagcount = 0;
			$tags = Array();
			
			// Loop through tags
			foreach( $rsp_obj['who']['tags']['tag'] as $tag )
			{
				if($count==0){
					$mintagcount=$tag['count'];
					$maxtagcount=$tag['count'];					
				}
				
				if($mintagcount>$tag['count'])
					$mintagcount=$tag['count'];
					
				if($maxtagcount<$tag['count'])
					$maxtagcount=$tag['count'];	
									
				$aux[0]=$tag['count'];
				$aux[1]=$tag['_content'];
				$tags[$count]=$aux;
				
				$count++;
			}
			
			if($maxtagcount==$mintagcount)
				$mintagcount=$maxtagcount-1;

			foreach($tags as $tag){
				// Determine font size
				$fontsize=round(((($maxfontsize-$minfontsize)/($maxtagcount-$mintagcount))*($tag[0]-$mintagcount))+$minfontsize);
				// Write out link
				$data .= "<a href=\"" . $photourl ."tags/".$tag[1]."\" target=\"" .$target. "\"  rel=\"nofollow\" style=\"font-size: ".$fontsize."px;\">".$tag[1]."</a>&nbsp;\n";
			}
		}
		else {
			// Check if error code given and display
			if ( $rsp_obj['stat'] == 'fail' ) {
				$data .= "Error code: ".$rsp_obj['code']." ".$rsp_obj['message']."<br />\n";
			}
			else {
				$data .= "Unable to connect to flickr";
			}
		}
		$data .= "</div>";
		$data .= $after_widget . "\n";
		
		return $data;
	}
	
	// main widget function
	function widget_flickrtagcloud($args) {
		// Reset ouput
		$data = '';
		
		if ($cache == -1) {	
			// Get data from flickr
			$data = get_flickr_data($args);		
		} else {
			if (!is_file(CACHEFILE) || (filesize(CACHEFILE) == 0) || (filemtime(CACHEFILE) < (time() - $cache))) {				
				// Get data from flickr
				$data = get_flickr_data($args);
	
				// Write cache file
				$fh = fopen(CACHEFILE, "w");
				fwrite($fh, $file_contents);
				fclose($fh);
			}
			else if (file_exists(CACHEFILE)) {	
				// Fetch the info
				$data = file_get_contents_curl(CACHEFILEURL);
			}
		}
		
		// Print output to page
		echo $data;
	}
	
	// control panel
	function widget_flickrtagcloud_control() {
		$options = $newoptions = get_option('widget_flickrtagcloud');
		if ( $_POST["flickrtagcloud-submit"] ) {
			$newoptions['title'] = trim(strip_tags(stripslashes($_POST["flickrtagcloud-title"])));
			$newoptions['user'] = trim(strip_tags(stripslashes($_POST["flickrtagcloud-user"])));
			$newoptions['minfontsize'] = trim(strip_tags(stripslashes($_POST["flickrtagcloud-minfontsize"])));
			$newoptions['maxfontsize'] = trim(strip_tags(stripslashes($_POST["flickrtagcloud-maxfontsize"])));
			$newoptions['key'] = trim(strip_tags(stripslashes($_POST["flickrtagcloud-key"])));
			$newoptions['maxcount'] = trim(strip_tags(stripslashes($_POST["flickrtagcloud-maxcount"])));
			$newoptions['cache'] = trim(strip_tags(stripslashes($_POST["flickrtagcloud-cache"])));
			$newoptions['target'] = trim(strip_tags(stripslashes($_POST["flickrtagcloud-target"])));
			
			//Remove previous cache file if it exist in order to apply the new configuration.
			if (is_file(CACHEFILE)) {
				unlink(CACHEFILE);
			}	
		}
		
		if ( $options != $newoptions ) {
			$options = $newoptions;
			update_option('widget_flickrtagcloud', $options);
		}
		$title = htmlspecialchars($options['title'], ENT_QUOTES);
		$user = htmlspecialchars($options['user'], ENT_QUOTES);
		$minfontsize = htmlspecialchars($options['minfontsize'], ENT_QUOTES);
		$maxfontsize = htmlspecialchars($options['maxfontsize'], ENT_QUOTES);
		$key = htmlspecialchars($options['key'], ENT_QUOTES);
		$maxcount = htmlspecialchars($options['maxcount'], ENT_QUOTES);
		$cache = htmlspecialchars($options['cache'], ENT_QUOTES);
		$target = htmlspecialchars($options['target'], ENT_QUOTES);
		
		// Defaults
		if(!$minfontsize or $minfontsize<1) $minfontsize = DEFAULT_MINFONTSIZ;
		if(!$maxfontsize or $maxfontsize<1) $maxfontsize = DEFAULT_MAXFONTSIZ;
		if(!$maxcount or $maxcount<-1) $maxcount = DEFAULT_MAXCOUNT;
		if(!$cache or $cache<-1) $cache = DEFAULT_CACHETIME; 
		if(!target) $target = DEFAULT_TARGET;
		?>
		<table>
		<tr><td nowrap="nowrap"><strong>Widget Title:</strong></td>
		<td><input id="flickrtagcloud-title" name="flickrtagcloud-title" type="text" size="20" value="<?php echo $title; ?>" /></td>
		<td style="font-size:0.9em">Optional</td>
		</tr>
		<tr><td nowrap="nowrap"><strong>API Key:</strong></td>
		<td><input id="flickrtagcloud-key" name="flickrtagcloud-key" type="text" size="20" value="<?php echo $key; ?>" /></td>
		<td style="font-size:0.9em">Go to <a href="http://www.flickr.com/services/api/keys/" target="_blank">http://www.flickr.com/services/api/keys/</a> to get an API key</td>
		</tr>
		<tr><td nowrap="nowrap"><strong>Flickr ID:</strong></td>
		<td><input id="flickrtagcloud-user" name="flickrtagcloud-user" type="text" size="20" value="<?php echo $user; ?>" /></td>
		<td style="font-size:0.9em">Use <a href="http://idgettr.com/" target="idgettr">http://idgettr.com/</a> to find your ID</td>
		</tr>
		<tr><td nowrap="nowrap"><strong>Cache Time:</strong></td>
		<td><input id="flickrtagcloud-cache" name="flickrtagcloud-cache" type="text" size="6" value="<?php echo $cache; ?>" /></td>
		<td style="font-size:0.9em">The cache time value in seconds. ( -1 = disable, 604800 = a week )</td>
		</tr>
		<tr><td nowrap="nowrap"><strong>Min Font Size:</strong></td>
		<td><input id="flickrtagcloud-minfontsize" name="flickrtagcloud-minfontsize" type="text" size="2" value="<?php echo $minfontsize; ?>" /></td>
		<td style="font-size:0.9em">The minimum size in pixels to be used for the tag links</td>
		</tr>
		<tr><td nowrap="nowrap"><strong>Max Font Size:</strong></td>
		<td><input id="flickrtagcloud-maxfontsize" name="flickrtagcloud-maxfontsize" type="text" size="2" value="<?php echo $maxfontsize; ?>" /></td>
		<td style="font-size:0.9em">The maximum size in pixels to be used for the tag links</td>
		</tr>
		<tr><td nowrap="nowrap"><strong>Max Count:</strong></td>
		<td><input id="flickrtagcloud-maxcount" name="flickrtagcloud-maxcount" type="text" size="2" value="<?php echo $maxcount; ?>" /></td>
		<td style="font-size:0.9em">The maximum number of tag links to be displayed. ( -1 = unlimited )</td>
		</tr>
		<tr><td nowrap="nowrap"><strong>Link Target:</strong></td>
		<td><select id="flickrtagcloud-target" name="flickrtagcloud-target">
			<option value="_blank" <?php if($target=='_blank')	echo("selected=\"selected\""); ?> >_blank</option>
			<option value="_self" <?php if($target=='_self')	echo("selected=\"selected\""); ?> >_self</option>
			<option value="_parent" <?php if($target=='_parent')	echo("selected=\"selected\""); ?> >_parent</option>
			<option value="_top" <?php if($target=='_top')	echo("selected=\"selected\""); ?> >_top</option>
		</select></td>
		<td style="font-size:0.9em">The target attribute for the anchor tags.  Allows you to set where the links open.</td>
		</tr>
		</table>
		<input type="hidden" id="flickrtagcloud-submit" name="flickrtagcloud-submit" value="1" />
		<?
	}
	
	register_sidebar_widget(array('Flickr Tag Cloud', 'widgets'), 'widget_flickrtagcloud');
	register_widget_control(array('Flickr Tag Cloud', 'widgets'), 'widget_flickrtagcloud_control', 600, 330);
}

// Tell Dynamic Sidebar about our new widget and its control
add_action('plugins_loaded', 'widget_flickrtagcloud_init');

?>
